#include "ipsetup.h"
#include <QNetworkInterface>
#include <QtNetwork>
#include <QHostAddress>
#include <iostream>

#define LOGME std::cout << "We made it to line " << __LINE__ << " of file " << __FILE__ << std::endl;

const QString NBurnSocket::magicPassString("1414");

NBurnSocket::NBurnSocket(const QHostAddress &localAddr, quint16 localPort, QObject * parent)
    : QUdpSocket(parent), m_remotePort(UDP_NETBURNERID_PORT)
{
    this->bind(localAddr, localPort, QUdpSocket::ShareAddress | QUdpSocket::ReuseAddressHint);
}

NBurnSocket::~NBurnSocket()
{

}


void NBurnSocket::SendRequest()
{
    ToFromPcStruct tfpc;
    tfpc.m_dwKeyValue = htonl( VERIFY_FROM_PC_TO_NDK );
    tfpc.m_bAction = NBAUTO_READ;

    int ret = writeDatagram( (const char *)&tfpc, sizeof(tfpc),
                   QHostAddress::Broadcast, m_remotePort);

    if (ret < 0) {
        std::cout << "Error sending: " << error() << std::endl;
    }
}

void NBurnSocket::SendVerify(const QList<NBurnDevice> &devList)
{
    quint8 buffer[1500];
    ToFromPcStruct *ptfpc = (ToFromPcStruct *)buffer;
    ptfpc->m_dwKeyValue = htonl(VERIFY_FROM_PC_TO_NDKV2);
    ptfpc->m_bAction = 'V';

    quint8 * p = buffer + 5;
    for (int i = 0; i < devList.size(); i++) {
        const QList<NBurnInterface> &m_if = devList[i].m_if;
        for (int j = 0; j < m_if.size(); j++) {
            const ConfigRecord &cr = m_if[j].m_cf;
            if (ConnectedDirectly(cr, m_if[j].m_received_from.toIPv4Address())) {
                *p++ = cr.mac_address[4];
            }
            else {
                *p++ = cr.mac_address[4] ^ 0x80;
            }
            *p++ = cr.mac_address[5];
        }
    }
    *p++ = 0;
    *p++ = 0;
    *p++ = 0;
    *p++ = 0;

    int len = (p + 1) - buffer;

    writeDatagram( (const char *)buffer, len,
                       QHostAddress::Broadcast, m_remotePort);
}

void NBurnSocket::SendNewConfig(const ConfigRecord &cr)
{
    ToFromPcStruct tfpc;
    tfpc.m_dwKeyValue = htonl(VERIFY_FROM_PC_TO_NDK);
    tfpc.m_bAction = NBAUTO_WRITE;
    memcpy(&tfpc.m_cf,&cr,sizeof(ConfigRecord));

    writeDatagram( (const char *)&tfpc, sizeof(tfpc),
                   QHostAddress::Broadcast, m_remotePort);
}

void NBurnSocket::SendNewConfigwPass(const ConfigRecord &cr,
                                     const QString &name, const QString &pass)
{
    QByteArray data;
    ToFromPcStruct tfpc;
    tfpc.m_dwKeyValue = htonl(VERIFY_FROM_PC_TO_NDK);
    tfpc.m_bAction = NBAUTO_WRITE;
    memcpy(&tfpc.m_cf,&cr,sizeof(ConfigRecord));

    data.append((char *)&tfpc,sizeof(ToFromPcStruct));

    //Add the magic string for password protection
    data.append(magicPassString);

    QString buffer(pass);
    buffer = buffer.left(19);
    data.append(buffer);
    for (int i = buffer.length(); i < 20; i++) {
        data.append('\0');
    }
    data.append(name);
    data.append('\0');
    writeDatagram(data, QHostAddress::Broadcast, m_remotePort);
}

IPSetup::IPSetup(QObject *parent) :
    QObject(parent), m_timerCycle(0), m_inProgress(false)
{
    QList<QHostAddress> addrs = QNetworkInterface::allAddresses();
    for (int i = 0; i < addrs.size(); i++) {
        // Since the NNDK only supports IPv4 at the moment...
        if (addrs[i].protocol() == QAbstractSocket::IPv4Protocol) {
            m_socks.append(new NBurnSocket(addrs[i],UDP_NETBURNERID_PORT + 1));
            connect(m_socks.last(), SIGNAL(readyRead()), this, SLOT(ReadPendingDatagrams()));
        }
    }
}


IPSetup::~IPSetup()
{
    for (int i = 0; i < m_socks.size(); i++) {
        delete m_socks[i];
    }
    m_socks.clear();
}


void IPSetup::FindDevices()
{
    if (m_inProgress) { return; }
    m_inProgress = true;

    m_devices.clear();
    for (int i = 0; i < m_socks.size(); i++) {
        m_socks[i]->SendRequest();
    }

    startTimer(250);
}

void IPSetup::ProcessPacket(const QByteArray &pkt, const QHostAddress &sender)
{
    QString Device_Name("");
    QString Device_App("");
    bool    bPass = false;

    // A Configuration Record Packet contains a bit of header and then the config record
    // and then after the config record, the null terminated Device Name, followed by the
    // null terminated Application Name, and potentially followed by the PASSNEEDED flag
    if (pkt.size() > sizeof(ToFromPcStruct)) {
        // finde the null char after the end of the config record
        int endOfName = pkt.indexOf('\0', sizeof( ToFromPcStruct ));
        int clen = endOfName - sizeof( ToFromPcStruct );
        if (clen < 0) {
            clen = 0;
        }
        // Device name starts after the config record, and is null terminated
        Device_Name = QString::fromUtf8(pkt.mid(sizeof(ToFromPcStruct),clen).data());
        if (pkt.size() > (endOfName + 2)) {
            int endOfApp = pkt.indexOf('\0',endOfName+1);
            clen = endOfApp - (endOfName + 1);
            if (clen < 0) {
                clen = 0;
            }
            Device_App = QString::fromUtf8(pkt.mid(endOfName+1,clen).data());
            {
                QString extra(pkt.mid(endOfApp+2,pkt.size()));
                if (extra.indexOf(QString("PASSNEEDED")) >= 0) {
                    bPass = true;
                }
            }

            if (Device_App == ":Unknown") {
                Device_App = "";
            }
        }
        else {
            Device_App = "";
        }
    }

    ToFromPcStruct *tfpc = (ToFromPcStruct *)pkt.data();

    if ( (tfpc->m_dwKeyValue == htonl(VERIFY_FROM_NDK_TO_PC)) ) {
        QString buffer;
        quint32 dwip = 0;
        dwip =              tfpc->m_cf.ip_Addr[0];
        dwip = (dwip << 8) |tfpc->m_cf.ip_Addr[1];
        dwip = (dwip << 8) |tfpc->m_cf.ip_Addr[2];
        dwip = (dwip << 8) |tfpc->m_cf.ip_Addr[3];
        if (sender.toIPv4Address() != dwip) {
            buffer.sprintf(
                "MAC: [%02x-%02X-%02X-%02X-%02X-%02X]    IP:%d.%d.%d.%d DHCP:%d.%d.%d.%d  ",
                tfpc->m_cf.mac_address[0],
                tfpc->m_cf.mac_address[1],
                tfpc->m_cf.mac_address[2],
                tfpc->m_cf.mac_address[3],
                tfpc->m_cf.mac_address[4],
                tfpc->m_cf.mac_address[5],
                tfpc->m_cf.ip_Addr[0],
                tfpc->m_cf.ip_Addr[1],
                tfpc->m_cf.ip_Addr[2],
                tfpc->m_cf.ip_Addr[3],
                (sender.toIPv4Address() >> 24),
                (sender.toIPv4Address() >> 16) & 0xFF,
                (sender.toIPv4Address() >> 8) & 0xFF,
                sender.toIPv4Address() & 0xFF);
//                sa.sin_addr.S_un.S_un_b.s_b2,
//                sa.sin_addr.S_un.S_un_b.s_b3,
//                sa.sin_addr.S_un.S_un_b.s_b4,
        }
        else
        {
            buffer.sprintf(
                "MAC: [%02x-%02X-%02X-%02X-%02X-%02X]    IP:%d.%d.%d.%d               ",
                tfpc->m_cf.mac_address[0],
                tfpc->m_cf.mac_address[1],
                tfpc->m_cf.mac_address[2],
                tfpc->m_cf.mac_address[3],
                tfpc->m_cf.mac_address[4],
                tfpc->m_cf.mac_address[5],
                tfpc->m_cf.ip_Addr[0],
                tfpc->m_cf.ip_Addr[1],
                tfpc->m_cf.ip_Addr[2],
                tfpc->m_cf.ip_Addr[3]);
        }
        buffer.append(Device_Name);
        buffer.append(':');
        buffer.append(Device_App);
//        std::cout << buffer.toStdString() << std::endl;

        quint32 dwip_used;
        dwip_used =              tfpc->m_cf.m_dwIP_used[0];
        dwip_used = (dwip_used << 8) |tfpc->m_cf.m_dwIP_used[1];
        dwip_used = (dwip_used << 8) |tfpc->m_cf.m_dwIP_used[2];
        dwip_used = (dwip_used << 8) |tfpc->m_cf.m_dwIP_used[3];
        NBurnInterface intf;
        intf.m_received_from = sender;
        intf.m_directConnect = sender.toIPv4Address() == dwip_used;

        intf.m_ipMethod = UsingAutoIP(tfpc->m_cf)
                ? IPMethod_AutoIP
                : (UsingDHCP(tfpc->m_cf)
                   ? IPMethod_DHCP
                   : IPMethod_Static);


        memcpy(&intf.m_cf,&(tfpc->m_cf),sizeof(ConfigRecord));
        InsertInterface(m_devices, intf, Device_Name, Device_App, bPass);
    }
}


void IPSetup::ReadPendingDatagrams()
{
    for (int i = 0; i < m_socks.size(); i++) {
        while ((m_socks[i]->state() != QAbstractSocket::UnconnectedState)
               && (m_socks[i]->hasPendingDatagrams())) {
            QByteArray datagram;
            datagram.resize(m_socks[i]->pendingDatagramSize());
            QHostAddress sender;
            quint16 senderPort;

            m_socks[i]->readDatagram(datagram.data(), datagram.size(),
                                    &sender, &senderPort);

            ProcessPacket(datagram,sender);
        }
    }
}

void IPSetup::timerEvent(QTimerEvent *event)
{
    m_timerCycle++;
//    std::cout << "TimerCycle: " << m_timerCycle << std::endl;

    if ((m_timerCycle == 6) || (m_timerCycle == 12)) {
        for (int i = 0; i < m_socks.size(); i++) {
            m_socks[i]->SendVerify(m_devices);
        }
        ScanUpdate(m_devices, false);
    }

    if (m_timerCycle >= 18) {
        killTimer(event->timerId());
        m_timerCycle = 0;
//        for (int i = 0; i < m_devices.size(); i++) {
//            QList<NBurnInterface> &m_if = m_devices[i].m_if;
//            std::cout << "Device " << i << ": ";
//            printf("Core MAC[%02X-%02X-%02X-%02X-%02X-%02X]",
//                   m_devices[i].m_coreMac[0],
//                   m_devices[i].m_coreMac[1],
//                   m_devices[i].m_coreMac[2],
//                   m_devices[i].m_coreMac[3],
//                   m_devices[i].m_coreMac[4],
//                   m_devices[i].m_coreMac[5]
//                   );
//            std::cout << std::endl;
//            for (int j = 0; j < m_if.size(); j++) {
//                printf("    Intferface %d - MAC[%02X-%02X-%02X-%02X-%02X-%02X]",
//                               m_if[j].m_cf.typeof_if,
//                               m_if[j].m_cf.mac_address[0],
//                               m_if[j].m_cf.mac_address[1],
//                               m_if[j].m_cf.mac_address[2],
//                               m_if[j].m_cf.mac_address[3],
//                               m_if[j].m_cf.mac_address[4],
//                               m_if[j].m_cf.mac_address[5]
//                               );
//                std::cout << std::endl;

//            }
//        }
        m_inProgress = false;
        ScanUpdate(m_devices, true);
    }
}

void IPSetup::SendNewConfig(const ConfigRecord &cr)
{
    for (int i = 0; i < m_socks.size(); i++) {
        m_socks[i]->SendNewConfig(cr);
    }
}

void IPSetup::SendNewConfigwPass(const ConfigRecord &cr,
                                 const QString &name,
                                 const QString &pass)
{
    for (int i = 0; i < m_socks.size(); i++) {
        m_socks[i]->SendNewConfigwPass(cr, name, pass);
    }
}

void InsertInterface( QList<NBurnDevice> &devList, const NBurnInterface &intf,
                        const QString &Name, const QString &App,
                        bool needPass)
{
    const ConfigRecord &cr = intf.m_cf;
    int i = 0;
    for (i = 0; i < devList.size(); i++) {
        // If core macs are equal
        Q_ASSERT(i < devList.size());
        if (memcmp(cr.core_mac_address,devList[i].m_coreMac,6) == 0) {
            break;
        }
    }
    if (i < devList.size()) {
        QList<NBurnInterface> &m_if = devList[i].m_if;
        for (int j = 0; j < m_if.size(); j++) {
            Q_ASSERT(j < m_if.size());
            ConfigRecord &m_cf = m_if[j].m_cf;
            // If the record's mac matches, update it.
            if ((cr.mac_address[0] == m_cf.mac_address[0])
                && (cr.mac_address[1] == m_cf.mac_address[1])
                && (cr.mac_address[2] == m_cf.mac_address[2])
                && (cr.mac_address[3] == m_cf.mac_address[3])
                && (cr.mac_address[4] == m_cf.mac_address[4])
                && (cr.mac_address[5] == m_cf.mac_address[5]))
            {
                m_cf = cr;
                return;
            }
        }
        // Otherwise, add it
        m_if.append(intf);
        return;
    }

    NBurnDevice dev(cr.core_mac_address, Name, App, needPass);
    dev.m_if.append(intf);
    devList.append(dev);
}

bool ConnectedDirectly( const ConfigRecord &cr, unsigned long received_from )
{
    quint32 dwip;
    quint32 cdwip;
    if ( cr.direct_tx != 0xAA )
    {
        return true;
    }

    dwip = cr.m_dwIP_used[0];
    dwip = ( dwip << 8 ) | cr.m_dwIP_used[1];
    dwip = ( dwip << 8 ) | cr.m_dwIP_used[2];
    dwip = ( dwip << 8 ) | cr.m_dwIP_used[3];
    cdwip = dwip;
    dwip = htonl( dwip );
    if ( dwip == received_from )
    {
        return true;
    }

    dwip = cr.ip_Addr[0];
    dwip = ( dwip << 8 ) | cr.ip_Addr[1];
    dwip = ( dwip << 8 ) | cr.ip_Addr[2];
    dwip = ( dwip << 8 ) | cr.ip_Addr[3];
    dwip = htonl( dwip );
    if ( dwip == received_from )
    {
        return true;
    }


    return false;
}

bool UsingAutoIP(const ConfigRecord &cr)
{
    const unsigned char * pb = NULL;
    pb=cr.m_dwIP_used;


//    printf("%02X-%02X-%02X-%02X-%02X-%02X ",
//                   cr.mac_address[0],
//                    cr.mac_address[1],
//                    cr.mac_address[2],
//                    cr.mac_address[3],
//                    cr.mac_address[4],
//                    cr.mac_address[5]);
//    std::cout << "IP: ";
//    printf("%3d.%3d.%3d.%3d -- %s\r\n", pb[0],pb[1],pb[2],pb[3],
//            ((pb[0]==169) && (pb[1]==254)) ? "True" : "False");
    return ((pb[0]==169) && (pb[1]==254));
}

bool UsingDHCP(const ConfigRecord &cr)
{
    return !(( cr.ip_Addr[0] ) ||
            ( cr.ip_Addr[1] ) ||
            ( cr.ip_Addr[2] ) ||
            ( cr.ip_Addr[3] ) );
}

