#ifndef IPSETUP_H
#define IPSETUP_H

#include <QObject>
#include <QtNetwork>
#include <QHostAddress>



#define VERIFY_FROM_PC_TO_NDK   (0x4255524E) //BURN
#define VERIFY_FROM_PC_TO_NDKV2 (0x42555232) //BUR2
#define VERIFY_FROM_NDK_TO_PC (0x4E455442) //NETB
#define UDP_NETBURNERID_PORT (0x4E42) /*NB*/


#define NBAUTO_READ     'R'
#define NBAUTO_WRITE 'W'
#define NBAUTO_VERIFY   'V'
#define NBAUTO_ERR      (3)
#define NBAUTO_OK    (4)

#define htons qToBigEndian
#define htonl qToBigEndian

#pragma pack(1)
struct ConfigRecord
{
    quint32 m_Len;         // length of record
    quint8 ip_Addr[4];      /*The device IP Address */
    quint8 ip_Mask[4];      /*The IP Address Mask */
    quint8 ip_GateWay[4];   /*The address of the P gateway */
    quint8 ip_TftpServer[4];/*The address of the TFTP server to load data from for debugging */
    quint32 baud_rate;    /*The initial system Baud rate */
    unsigned char wait_seconds; /*The number of seconds to wait before booting */
    unsigned char bBoot_To_Application; /*True if we boot to the application, not the monitor */
    unsigned char bException_Action; /*What should we do when we have an exception? */
    unsigned char m_FileName[80]; /*The file name of the TFTP file to load */
    unsigned char mac_address[6]; /*The Ethernet MAC address */
    unsigned char ser_port;
    unsigned char ip_DNS[4];
    unsigned char core_mac_address[6];/*The Base unit MAC address */
    unsigned char typeof_if;
    unsigned char direct_tx;
    unsigned char m_dwIP_used[4];
    unsigned long m_dwMASK_used;
    unsigned long m_dwGate_used;
    unsigned long m_dwDNS_used;
    unsigned char m_bUnused[2];
    unsigned char m_Flags;
    unsigned char m_q_boot; /* True to boot without messages */
    unsigned short checksum;       /*A Checksum for this structure */
};

#define CONFIG_IF_ID_ETHERNET (0)
#define CONFIG_IF_ID_WIFI  (1)
#define CONFIG_IF_ID_ETHERNET2 (2)

enum NBurnIPMethod{
    IPMethod_Static,
    IPMethod_DHCP,
    IPMethod_AutoIP
};

struct ToFromPcStruct
{
    quint32 m_dwKeyValue;
    quint8 m_bAction;
    ConfigRecord m_cf;
};

#pragma pack()

struct NBurnInterface
{
    QHostAddress m_received_from;
    bool m_directConnect;
    NBurnIPMethod m_ipMethod;
    ConfigRecord m_cf;
};

struct NBurnDevice
{
    quint8 m_coreMac[6];
    QString m_DeviceName;
    QString m_DeviceApp;
    bool m_needPass;
    QList<NBurnInterface> m_if;
    NBurnDevice(const quint8 *mac, const QString &DeviceName, const QString &DeviceApp, bool needPass)
        :m_coreMac(),
          m_DeviceName(DeviceName),
          m_DeviceApp(DeviceApp),
          m_needPass(needPass),
          m_if()
    {
        memcpy(m_coreMac,mac,6);
    }
};

class NBurnSocket : public QUdpSocket
{
    Q_OBJECT

private:
    quint16 m_remotePort;
    static const QString magicPassString;

public:
    void SendRequest();
    void SendVerify( const QList<NBurnDevice> &devList );
    void SendNewConfig( const ConfigRecord &cr );
    void SendNewConfigwPass( const ConfigRecord &cr , const QString &name, const QString &pass);
    NBurnSocket( const QHostAddress & localAddr, quint16 localport, QObject * parent = 0 );
    ~NBurnSocket();

signals:

};

class IPSetup : public QObject
{
    Q_OBJECT

private:
    QList<NBurnSocket *> m_socks;
    QList<NBurnDevice> m_devices;
    int m_timerCycle;
    bool m_inProgress;

public:
    explicit IPSetup(QObject *parent = 0);
    ~IPSetup();

    void FindDevices();
    void SendNewConfig(const ConfigRecord &cr);
    void SendNewConfigwPass( const ConfigRecord &cr , const QString &name, const QString &pass);

protected:
    virtual void timerEvent(QTimerEvent *event);

private:
    void ProcessPacket(const QByteArray &pkt, const QHostAddress &sender);

signals:
    void ScanUpdate(const QList<NBurnDevice> &devList, bool finished);

public slots:
    

private slots:
    void ReadPendingDatagrams();
};

void InsertInterface( QList<NBurnDevice> &devList, const NBurnInterface &intf,
                      const QString &Name,      const QString &App,
                      bool needPass);

bool ConnectedDirectly( const ConfigRecord &cr, unsigned long received_from );
bool UsingAutoIP(const ConfigRecord &cr);
bool UsingDHCP(const ConfigRecord &cr);

#endif // IPSETUP_H
