#include "ipsetupwindow.h"
#include "ui_ipsetupwindow.h"
#include <iostream>
#include <QtGlobal>
#include <QDialog>
#include <QDesktopServices>

QPixmap *IPSetupTreeItem::statusIcons = NULL;
int IPSetupTreeItem::sortColumn = 1;
Qt::SortOrder IPSetupTreeItem::sortOrder = Qt::AscendingOrder;




IPSetupWindow::IPSetupWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::IPSetupWindow),
    m_viewHeader(NULL),
    m_finder(new IPSetup),
    m_devModel(NULL),
    m_clearModelNext(true)
{
    ui->setupUi(this);
    QIcon mainIcon(":/ipsetup.ico");
    this->setWindowIcon(mainIcon);
    setWindowFlags(windowFlags() ^ Qt::WindowMaximizeButtonHint);

    connect(m_finder,&IPSetup::ScanUpdate,this,&IPSetupWindow::ScanUpdate);


    IPSetupTreeItem::initIcons(":/bit_tree.bmp");

    QList<NBurnDevice> devList;
    m_devModel = new IPSetupTreeModel(devList);
    ui->devListTreeView->setModel(m_devModel);
    ui->devListTreeView->setWindowTitle("NDK Devices");
    ui->devListTreeView->show();
    SetWifiInputState(false);

    ui->baudrateComboBox->addItem("300",300);
    ui->baudrateComboBox->addItem("600",600);
    ui->baudrateComboBox->addItem("1200",1200);
    ui->baudrateComboBox->addItem("2400",2400);
    ui->baudrateComboBox->addItem("4800",4800);
    ui->baudrateComboBox->addItem("9600",9600);
    ui->baudrateComboBox->addItem("19200",19200);
    ui->baudrateComboBox->addItem("38400",38400);
    ui->baudrateComboBox->addItem("57600",57600);
    ui->baudrateComboBox->addItem("115200",115200);

    ui->monitorPortComboBox->addItem("Uart 0", 0);
    ui->monitorPortComboBox->addItem("Uart 1", 1);

    ui->trapModeComboBox->addItem("Reboot", 0);
    ui->trapModeComboBox->addItem("Halt", 1);
    ui->trapModeComboBox->addItem("Quiet Reboot", 2);

    SetTreeColumnWidths();

    QHeaderView *viewHeader = ui->devListTreeView->header();
    viewHeader->setSectionsMovable(false);
    viewHeader->setSortIndicator(1,Qt::AscendingOrder);
    connect(viewHeader,&QHeaderView::sortIndicatorChanged,this,&IPSetupWindow::viewHeader_sortChanged);
    on_searchButton_clicked();
}

IPSetupWindow::~IPSetupWindow()
{
    delete m_finder;
    delete ui;
}

void IPSetupWindow::resizeEvent(QResizeEvent *event)
{
    QMainWindow::resizeEvent(event);

    QGroupBox *gb = ui->devListGroupBox;
    int newWidth = this->width() - 9;
    int d_width = newWidth - gb->width();
    gb->setGeometry(gb->x(),gb->y(),newWidth,gb->height());
    QTreeView *tv = ui->devListTreeView;
    tv->setGeometry(tv->x(),tv->y(),gb->width()-20,tv->height());

    QPushButton *pb = ui->browserButton;
    pb->setGeometry(pb->x()+d_width,pb->y(),pb->width(),pb->height());
    pb = ui->setConfigButton;
    pb->setGeometry(pb->x()+(d_width/2),pb->y(),pb->width(),pb->height());

    gb = ui->netSetGroupBox;
    gb->setGeometry(gb->x(),gb->y(),newWidth,gb->height());
    QWidget *lw = ui->horizontalLayoutWidget;
    lw->setGeometry(lw->x(),lw->y(),gb->width()-9,gb->height()-9);

    gb = ui->advancedBox;
    gb->setGeometry(gb->x(),gb->y(),newWidth,gb->height());
    lw = ui->gridLayoutWidget;
    lw->setGeometry(lw->x(),lw->y(),gb->width()-75,gb->height()-85);

    gb = ui->devListGroupBox;
    int newHeight = this->height() - 259;
    int d_height = newHeight - gb->height();
    gb->setGeometry(gb->x(),gb->y(),gb->width(),newHeight);
    tv->setGeometry(tv->x(),tv->y(),tv->width(),newHeight-55);

    QWidget *qw = ui->searchButton;
    qw->move(qw->x(),qw->y()+d_height);
    qw = ui->setConfigButton;
    qw->move(qw->x(),qw->y()+d_height);
    qw = ui->browserButton;
    qw->move(qw->x(),qw->y()+d_height);
    qw = ui->netSetGroupBox;
    qw->move(qw->x(),qw->y()+d_height);
    qw = ui->advancedBox;
    qw->move(qw->x(),qw->y()+d_height);
}

void IPSetupWindow::keyPressEvent(QKeyEvent * event)
{
    if (event->key()== Qt::Key_Escape) {
        close();
    }
    else {
        QMainWindow::keyPressEvent(event);
    }
}

//void IPSetupWindow::on_closeButton_clicked()
//{
////    exit(0);
//}

void IPSetupWindow::on_searchButton_clicked()
{
    m_finder->FindDevices();
    ui->searchButton->setDisabled(true);
    m_clearModelNext = true;
}

void IPSetupWindow::viewHeader_sortChanged(int logicalIndex, Qt::SortOrder order)
{
    if (!logicalIndex) { return; }
    ui->devListTreeView->collapseAll();

    ui->devListTreeView->setSortingEnabled(false);

    m_devModel->sort(logicalIndex,order);

    ui->devListTreeView->setModel(NULL);
    ui->devListTreeView->setModel(m_devModel);
    SetTreeColumnWidths();
    ui->devListTreeView->setSortingEnabled(true);

    for (int i = 0; i < m_devModel->rowCount(); i++) {
        QModelIndex index = m_devModel->index(i,0);
        ui->devListTreeView->update(index);
    }
}

void IPSetupWindow::ScanUpdate(const QList<NBurnDevice> &devList, bool finished)
{
    if (finished) {
        ui->searchButton->setDisabled(false);
        m_clearModelNext = true;
    }
    if (m_devModel) {
        if (m_clearModelNext) {
            ui->devListTreeView->collapseAll();
            ui->devListTreeView->clearSelection();
            delete m_devModel;
            m_devModel = new IPSetupTreeModel(devList);
            ui->devListTreeView->setModel(m_devModel);
            ui->devListTreeView->repaint();
        }
        else {
            m_devModel->updateModel(devList);
        }
    }
    else {
        m_devModel = new IPSetupTreeModel(devList);
        ui->devListTreeView->setModel(m_devModel);
    }
    m_clearModelNext = false;
//    QFont font = ui->devListTreeView->font();
//    font.setPointSize(8);
//    ui->devListTreeView->setFont(font);

    SetTreeColumnWidths();
}

void IPSetupWindow::SetTreeColumnWidths()
{
    ui->devListTreeView->setColumnWidth(0,60);
    ui->devListTreeView->setColumnWidth(1,70);
    ui->devListTreeView->setColumnWidth(2,105);
    ui->devListTreeView->setColumnWidth(3,80);
    ui->devListTreeView->setColumnWidth(4,50);
    ui->devListTreeView->setColumnWidth(5,50);
}


void IPSetupTreeItem::initIcons(const QString &iconFile)
{
    statusIcons = new QPixmap(iconFile);

//    std::cout << iconFile.toStdString() << " height: " << statusIcons->height()
//             << "  width: " << statusIcons->width()
//             << std::endl;
}

void IPSetupTreeItem::initIcons(const char *iconFile)
{
    statusIcons = new QPixmap(iconFile);

//    bool ret = statusIcons->load(iconFile);
//    std::cout << "Load " << (ret ? "success" : "failure") << std::endl;
//    std::cout << iconFile << " height: " << statusIcons->height()
//             << "  width: " << statusIcons->width()
//             << std::endl;
}

// order is Platform - MAC - IP Addr - Method - App Name
IPSetupTreeItem::IPSetupTreeItem(const NBurnDevice &dev, const NBurnInterface &intf,
                                 IPSetupTreeItem *parent)
    : m_parent(parent), m_cf(intf.m_cf), m_needPass(dev.m_needPass)
{
    SetItemData(dev,intf);
    if (parent) { parent->m_childItems.append(this); }
}

IPSetupTreeItem::IPSetupTreeItem(const QString &plat, const unsigned char *mac,
                                 const QString &app, bool needPass,
                                 IPSetupTreeItem *parent)
    :m_parent(parent), m_needPass(needPass)
{
    QString buffer;
    buffer.sprintf("%02X-%02X-%02X-%02X-%02X-%02X",
                   mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    m_itemData << statusIcons->copy(16, 0, 16, 16) << plat << buffer << "" << "" << "" << app;

    if (parent) { parent->m_childItems.append(this); }
}

IPSetupTreeItem::IPSetupTreeItem(const QList<QVariant> &qvl)
    :m_parent(0), m_needPass(false)
{
    m_itemData = qvl;
}

IPSetupTreeItem::~IPSetupTreeItem()
{
    qDeleteAll(m_childItems);
}

void IPSetupTreeItem::appendChild(IPSetupTreeItem *child)
{
    m_childItems.append(child);
    child->m_parent = this;
}

IPSetupTreeItem *IPSetupTreeItem::child(int row)
{
    return m_childItems.value(row);
}

int IPSetupTreeItem::childCount() const
{
    return m_childItems.count();
}

int IPSetupTreeItem::columnCount() const
{
    return m_itemData.count();
}

QVariant IPSetupTreeItem::data(int column) const
{
    return m_itemData.value(column);
}

IPSetupTreeItem *IPSetupTreeItem::parent()
{
    return m_parent;
}

int IPSetupTreeItem::row() const
{
    if (m_parent) {
        return m_parent->m_childItems.indexOf(const_cast<IPSetupTreeItem *>(this));
    }

    return 0;
}

void IPSetupTreeItem::SetItemData(const NBurnDevice &dev, const NBurnInterface &intf)
{
    QString buffer;

    if (m_cf.typeof_if == CONFIG_IF_ID_WIFI) {
        if (intf.m_directConnect) {
            m_itemData << statusIcons->copy(64, 0, 16, 16);
        }
        else {
            m_itemData << statusIcons->copy(80, 0, 16, 16);
        }
    }
    else {
        if (intf.m_directConnect) {
            m_itemData << statusIcons->copy(32, 0, 16, 16);
        }
        else {
            m_itemData << statusIcons->copy(48, 0, 16, 16);
        }
    }

    m_itemData << dev.m_DeviceName;
    buffer.sprintf("%02X-%02X-%02X-%02X-%02X-%02X",
                   m_cf.mac_address[0],
                    m_cf.mac_address[1],
                    m_cf.mac_address[2],
                    m_cf.mac_address[3],
                    m_cf.mac_address[4],
                    m_cf.mac_address[5]);
    m_itemData << buffer;
    buffer.clear();

    buffer.sprintf("%d.%d.%d.%d",
                   m_cf.m_dwIP_used[0],
                    m_cf.m_dwIP_used[1],
                    m_cf.m_dwIP_used[2],
                    m_cf.m_dwIP_used[3]);
    m_itemData << buffer;
    buffer.clear();

    switch(m_cf.typeof_if) {
    case CONFIG_IF_ID_ETHERNET: m_itemData << "Eth"; break;
    case CONFIG_IF_ID_ETHERNET2: m_itemData << "Eth 2"; break;
    case CONFIG_IF_ID_WIFI: m_itemData << "Wifi"; break;
    }

    switch(intf.m_ipMethod) {
    case IPMethod_Static: m_itemData << "Static"; break;
    case IPMethod_AutoIP: m_itemData << "AutoIP"; break;
    case IPMethod_DHCP: m_itemData << "DHCP"; break;
    }

    m_itemData << dev.m_DeviceApp;

}

bool IPSetupTreeItem::compare(const IPSetupTreeItem *lhs, const IPSetupTreeItem *rhs)
{
    bool lessThan = false;
    if (lhs && rhs) {
        switch(sortColumn) {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6: {
            QString lstr = lhs->m_itemData[sortColumn].toString();
            QString rstr = rhs->m_itemData[sortColumn].toString();
            lessThan = QString::compare(lstr,rstr, Qt::CaseInsensitive) < 0;
            break;
        }
        }
    }
    if (sortOrder == Qt::AscendingOrder) {
        return lessThan;
    }
    else { return !lessThan; }
}

IPSetupTreeModel::~IPSetupTreeModel()
{
    delete m_rootItem;
}

int IPSetupTreeModel::columnCount(const QModelIndex &parent) const
{
    return m_rootItem->columnCount();
    if (parent.isValid()) {
        return static_cast<IPSetupTreeItem *>(parent.internalPointer())->columnCount();
    }
    else {
        return m_rootItem->columnCount();
    }
}

bool IPSetupTreeModel::hasChildren(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        if (flags(parent) & 0x80) {
            return false;
        }
        return rowCount(parent);
    }
    return m_rootItem->childCount();
}

QVariant IPSetupTreeModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid()) { return QVariant(); }
    if ((index.column() == 0) && (role == Qt::DecorationRole)) {
        IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());
        return item->data(index.column());
    }
    if (role != Qt::DisplayRole) { return QVariant(); }

    IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());
    return item->data(index.column());
}

Qt::ItemFlags IPSetupTreeModel::flags(const QModelIndex &index) const
{
    if (!index.isValid()) { return 0; }
    IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());

    if (item->childCount() == 0) {
        return Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemNeverHasChildren;
    }
    else {
        return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
    }
}

QVariant IPSetupTreeModel::headerData(int section, Qt::Orientation orientation,
                                      int role) const
{
    if ((orientation == Qt::Horizontal) && (role == Qt::DisplayRole)) {
        return m_rootItem->data(section);
    }

    return QVariant();
}

QModelIndex IPSetupTreeModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent)) { return QModelIndex(); }

    IPSetupTreeItem *parentItem;

    if (!parent.isValid()) {
        parentItem = m_rootItem;
    }
    else {
        parentItem = static_cast<IPSetupTreeItem *>(parent.internalPointer());
    }

    IPSetupTreeItem *childItem = parentItem->child(row);
    if (childItem) {
        return createIndex(row, column, childItem);
    }
    else {
        return QModelIndex();
    }
}

QModelIndex IPSetupTreeModel::parent(const QModelIndex &index) const
{
    if (!index.isValid()) { return QModelIndex(); }

    IPSetupTreeItem *childItem = static_cast<IPSetupTreeItem *>(index.internalPointer());
    IPSetupTreeItem *parentItem = childItem->parent();

    if (parentItem == m_rootItem) { return QModelIndex(); }

    return createIndex(parentItem->row(), 0, parentItem);
}

int IPSetupTreeModel::rowCount(const QModelIndex &parent) const
{
    IPSetupTreeItem *parentItem;
//    if (parent.column() > 0) { return 0; }

    if (!parent.isValid()) {
        parentItem = m_rootItem;
    }
    else {
        parentItem = static_cast<IPSetupTreeItem *>(parent.internalPointer());
    }

    return parentItem->childCount();
}

IPSetupTreeModel::IPSetupTreeModel(const QList<NBurnDevice> &devList, QObject *parent)
    :QAbstractItemModel(parent)
{
    QList<QVariant> rootData;
    rootData << "" << "Platform" << "MAC" << "IP Address"  << "Intf" << "Method" << "App Name";
    m_rootItem = new IPSetupTreeItem(rootData);

    setupModelData(devList,m_rootItem);
}

void IPSetupTreeModel::setupModelData(const QList<NBurnDevice> &devList, IPSetupTreeItem *parent)
{
    for (int i = 0; i < devList.size(); i++) {
        const NBurnDevice &dev = devList[i];
        if (dev.m_if.size() > 1) {
            IPSetupTreeItem *parentItem;
            parentItem = new IPSetupTreeItem(dev.m_DeviceName,dev.m_coreMac,dev.m_DeviceApp,dev.m_needPass,parent);
            for (int j = 0; j < dev.m_if.size(); j++)
            {
                new IPSetupTreeItem(dev,dev.m_if[j],parentItem);
            }

        }
        else {
            new IPSetupTreeItem(dev,dev.m_if[0],parent);
        }
    }
}

void IPSetupTreeModel::updateModel(const QList<NBurnDevice> &devList)
{
    for (int i = 0; i < devList.size(); i++) {
        const NBurnDevice &dev = devList[i];
        QString buffer;
        const quint8 *mac = dev.m_if[0].m_cf.core_mac_address;
        buffer.sprintf("%02X-%02X-%02X-%02X-%02X-%02X",
                       mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
        for (int j = 0; j < m_rootItem->childCount(); j++) {
            quint8 *childMac = m_rootItem->child(j)->m_cf.core_mac_address;

            //if the core mac matches
            if (MacEqual(mac,childMac))
            {
                bool addEntry = false;
                childMac = m_rootItem->child(j)->m_cf.mac_address;
                for (int k = 0; k < dev.m_if.size(); k++) {
                    mac = dev.m_if[0].m_cf.mac_address;
                    //If the macs don't match...
                    if (!MacEqual(mac,childMac))
                    {
                        addEntry = true;
                        break;
                    }
                }
                if (addEntry) {
                    IPSetupTreeItem *item;
                    IPSetupTreeItem *child = m_rootItem->child(j);
                    IPSetupTreeItem *newChild =
                            new IPSetupTreeItem(dev.m_DeviceName,dev.m_coreMac,
                                                dev.m_DeviceApp,dev.m_needPass,
                                                m_rootItem);
                    m_rootItem->m_childItems[j] = newChild;
                    for (int k = 0; k < dev.m_if.size(); k++) {
                        item = new IPSetupTreeItem(dev,dev.m_if[k],newChild);
                        mac = item->m_cf.mac_address;
                        if (MacEqual(mac,childMac)) {
                            addEntry = false;
                        }
                    }
                    if (addEntry) {
                        newChild->appendChild(child);
                    }
                }
            }
            else if (buffer == m_rootItem->child(j)->data(2).toString())
            {
                IPSetupTreeItem *stepParent = m_rootItem->child(j);
                for (int x = 0; x < dev.m_if.size(); x++) {
                    bool addEntry = true;
                    mac = dev.m_if[x].m_cf.mac_address;
                    for (int y = 0; y < stepParent->childCount(); y++) {
                        childMac = stepParent->child(y)->m_cf.mac_address;
                        if (MacEqual(mac,childMac)) {
                            addEntry = false;
                            stepParent->child(y)->SetItemData(dev,dev.m_if[x]);
                            stepParent->child(y)->m_cf = dev.m_if[x].m_cf;
                            break;
                        }
                    }
                    if (addEntry) {
                        new IPSetupTreeItem(dev,dev.m_if[x],stepParent);
                    }
                }
            }
        }

    }
}

void IPSetupTreeModel::sort(int column, Qt::SortOrder order)
{
    IPSetupTreeItem::sortColumn = column;
    IPSetupTreeItem::sortOrder = order;

    if (column == 0) { return; }
    std::stable_sort(m_rootItem->m_childItems.begin(),m_rootItem->m_childItems.end(),IPSetupTreeItem::compare);

    for (int i = 0; i < m_rootItem->m_childItems.size(); i++) {
        IPSetupTreeItem *item = m_rootItem->m_childItems[i];
        if (item->m_childItems.size()) {
            std::stable_sort(item->m_childItems.begin(),item->m_childItems.end(),IPSetupTreeItem::compare);
        }
    }

    dataChanged(index(0,0),index(m_rootItem->childCount()-1,m_rootItem->columnCount()-1));
}

void IPSetupWindow::on_devListTreeView_activated(const QModelIndex &index)
{
    IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());
    if (item->childCount() > 0) {
        ui->devListTreeView->collapseAll();
        ui->devListTreeView->expand(index);
        QModelIndex childIndex = m_devModel->index(0,0,index.sibling(index.row(),0));
        QItemSelectionModel *selModel = ui->devListTreeView->selectionModel();
        selModel->select(childIndex,QItemSelectionModel::ClearAndSelect
                         | QItemSelectionModel::Rows);

        DoInputUpdate(childIndex);
    }
    else {
        // If the item's parent is the root node, collapse all rows
        if(m_devModel->parent(QModelIndex()) == index.parent()) {
            ui->devListTreeView->collapseAll();
        }

        DoInputUpdate(index);
    }
}

void IPSetupWindow::on_devListTreeView_clicked(const QModelIndex &index)
{
    on_devListTreeView_activated(index);
}

void IPSetupWindow::DoInputUpdate(const QModelIndex &index)
{
    IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());


    const ConfigRecord &cr = item->GetConfigRecord();
    ui->IPLineEdit->setText(cr.ip_Addr, 4);
    ui->networkMaskLineEdit->setText(cr.ip_Mask, 4);
    ui->dNSLineEdit->setText(cr.ip_DNS, 4);
    ui->gatewayLineEdit->setText(cr.ip_GateWay, 4);

    SetWifiInputState(cr.typeof_if == CONFIG_IF_ID_WIFI);

    ui->bootToAppCheckBox->setChecked(cr.bBoot_To_Application);
    ui->quietBootCheckBox->setChecked(cr.m_q_boot);

    if (item->data(1) == "MOD5441X") {
        ui->ESwitchEnableLabel->setVisible(true);
        ui->ESwitchEnableCheckBox->setVisible(true);
        ui->ESwitchEnableCheckBox->setChecked(cr.m_Flags & 0x02);
    }
    else {
        ui->ESwitchEnableLabel->setVisible(false);
        ui->ESwitchEnableCheckBox->setVisible(false);
    }

    QComboBox *baudCombo = ui->baudrateComboBox;
    baudCombo->setCurrentIndex(baudCombo->findData(htonl(cr.baud_rate)));

    QComboBox *monCombo = ui->monitorPortComboBox;
    monCombo->setCurrentIndex(monCombo->findData(cr.ser_port));

    QComboBox *trapCombo = ui->trapModeComboBox;
    trapCombo->setCurrentIndex(trapCombo->findData(cr.bException_Action));

    QString buffer;
    buffer.sprintf("%d",cr.wait_seconds);
    ui->bootDelayLineEdit->setText(buffer);

    ui->tFTPServerLineEdit->setText(cr.ip_TftpServer, 4);
    buffer.sprintf("%s",cr.m_FileName);
    ui->tFTPFileLineEdit->setText(buffer);
    if (cr.typeof_if == CONFIG_IF_ID_WIFI) {
        QList<QString> bufList = buffer.split(',');
        ui->sSIDLineEdit->setText(bufList[0].remove(0,5));
        ui->wifiKeyLineEdit->setText(bufList[1].remove(0,4));
    }
}


void IPSetupWindow::SetWifiInputState(bool state)
{
    ui->wifiKeyLabel->setVisible(state);
    ui->wifiKeyLineEdit->setVisible(state);
    ui->sSIDLabel->setVisible(state);
    ui->sSIDLineEdit->setVisible(state);
}

void IPSetupWindow::on_setConfigButton_clicked()
{
    ConfigRecord cr;
    QModelIndex index = ui->devListTreeView->currentIndex();
    IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());

    quint32 ipaddr = ui->IPLineEdit->getIPv4();

    cr = item->GetConfigRecord();
    cr.m_Len = htonl(sizeof(ConfigRecord));
    cr.ip_Addr[0] = ipaddr >> 24;
    cr.ip_Addr[1] = ipaddr >> 16;
    cr.ip_Addr[2] = ipaddr >> 8;
    cr.ip_Addr[3] = ipaddr >> 0;

    ipaddr = ui->networkMaskLineEdit->getIPv4();
    cr.ip_Mask[0] = ipaddr >> 24;
    cr.ip_Mask[1] = ipaddr >> 16;
    cr.ip_Mask[2] = ipaddr >> 8;
    cr.ip_Mask[3] = ipaddr >> 0;

    ipaddr = ui->dNSLineEdit->getIPv4();
    cr.ip_DNS[0] = ipaddr >> 24;
    cr.ip_DNS[1] = ipaddr >> 16;
    cr.ip_DNS[2] = ipaddr >> 8;
    cr.ip_DNS[3] = ipaddr >> 0;

    ipaddr = ui->tFTPServerLineEdit->getIPv4();
    cr.ip_TftpServer[0] = ipaddr >> 24;
    cr.ip_TftpServer[1] = ipaddr >> 16;
    cr.ip_TftpServer[2] = ipaddr >> 8;
    cr.ip_TftpServer[3] = ipaddr >> 0;

    QComboBox *baudCombo = ui->baudrateComboBox;
    cr.baud_rate = htonl(quint32(baudCombo->itemData(baudCombo->currentIndex()).toULongLong()));

    //FIXME - no validation
    cr.wait_seconds = ui->bootDelayLineEdit->text().toUShort();

    cr.bBoot_To_Application = (ui->bootToAppCheckBox->checkState()) ? 1 : 0;

    QComboBox *trapCombo = ui->trapModeComboBox;
    cr.bException_Action = htonl(quint32(trapCombo->itemData(trapCombo->currentIndex()).toULongLong()));

    strncpy((char *)cr.m_FileName,ui->tFTPFileLineEdit->text().toUtf8(),79);

    QComboBox *monCombo = ui->monitorPortComboBox;
    cr.ser_port = monCombo->itemData(monCombo->currentIndex()).toUInt();

    QString buffer;
    if (cr.typeof_if == CONFIG_IF_ID_WIFI) {
        buffer.clear();
        buffer.append("SSID:");
        buffer.append(ui->sSIDLineEdit->text());
        buffer.append(",WEP:");
        buffer.append(ui->wifiKeyLineEdit->text());

        strncpy((char *)cr.m_FileName,buffer.toUtf8(),79);
    }

    if (item->NeedPass()) {
        PasswordDialog pd;
        pd.exec();
        if (pd.result() == QDialog::Accepted) {
            QString uname = pd.usernameLineEdit->text();
            QString pw = pd.passwordLineEdit->text();

            m_finder->SendNewConfigwPass(cr,
                                         pd.usernameLineEdit->text(),
                                         pd.passwordLineEdit->text());
        }
        else {
            return;
        }
    }
    else {
        m_finder->SendNewConfig(cr);
    }

    on_searchButton_clicked();

}

void IPSetupWindow::on_browserButton_clicked()
{
    QModelIndex index = ui->devListTreeView->currentIndex();
    IPSetupTreeItem *item = static_cast<IPSetupTreeItem *>(index.internalPointer());
    if (!item) { return; }
    ConfigRecord cr = item->GetConfigRecord();

    QString buffer;
    buffer.sprintf("http://%d.%d.%d.%d/",
                   cr.m_dwIP_used[0],
                    cr.m_dwIP_used[1],
                    cr.m_dwIP_used[2],
                    cr.m_dwIP_used[3]);

    QDesktopServices::openUrl(buffer);
}
