#ifndef IPSETUPWINDOW_H
#define IPSETUPWINDOW_H

#include <QMainWindow>
#include <QImage>
#include <QHeaderView>
#include "ipsetup.h"
#include "passworddialog.h"

namespace Ui {
class IPSetupWindow;
}

class IPSetupTreeModel;

class IPSetupTreeItem
{
public:
    explicit IPSetupTreeItem(const NBurnDevice &dev, const NBurnInterface &cr,
                             IPSetupTreeItem *parent = 0);
    explicit IPSetupTreeItem(const QString &plat, const unsigned char *mac,
                             const QString &app, bool needPass,
                             IPSetupTreeItem *parent = 0);
    explicit IPSetupTreeItem(const QList<QVariant> &qvl);
    ~IPSetupTreeItem();

    static void initIcons(const QString &iconFile);
    static void initIcons(const char *iconFile);

    void appendChild(IPSetupTreeItem *child);

    IPSetupTreeItem *child(int row);
    int childCount() const;
    int columnCount() const;
    QVariant data(int column) const;
    int row() const;
    IPSetupTreeItem *parent();

    inline const ConfigRecord &GetConfigRecord() const { return m_cf; }
    inline bool NeedPass() const { return m_needPass; }

    static bool compare(const IPSetupTreeItem *lhs, const IPSetupTreeItem *rhs);

private:
    void SetItemData(const NBurnDevice &dev, const NBurnInterface &cr);

    static QPixmap *statusIcons;

    friend IPSetupTreeModel;

    QList<IPSetupTreeItem *> m_childItems;
    IPSetupTreeItem *m_parent;
    ConfigRecord m_cf;
    bool m_needPass;
    QList<QVariant> m_itemData;

    static int sortColumn;
    static Qt::SortOrder sortOrder;
};

class IPSetupTreeModel : public QAbstractItemModel
{
    Q_OBJECT

public:
    explicit IPSetupTreeModel(const QList<NBurnDevice> &devList, QObject *parent = 0);
    ~IPSetupTreeModel();

    QVariant data(const QModelIndex &index, int role) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex &index) const;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    bool hasChildren(const QModelIndex &parent) const;

    void sort(int column, Qt::SortOrder order);

    void updateModel(const QList<NBurnDevice> &devList);
private:
    void setupModelData(const QList<NBurnDevice> &devList, IPSetupTreeItem *parent);

    IPSetupTreeItem *m_rootItem;
    int m_sortColumn;
};

class IPSetupWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit IPSetupWindow(QWidget *parent = 0);
    ~IPSetupWindow();
    
private slots:
    void on_searchButton_clicked();

    void ScanUpdate(const QList<NBurnDevice> &devList, bool finished);

    void on_devListTreeView_activated(const QModelIndex &index);

    void on_devListTreeView_clicked(const QModelIndex &index);

    void on_setConfigButton_clicked();

    void viewHeader_sortChanged(int logicalIndex, Qt::SortOrder order);

    void resizeEvent(QResizeEvent *event);

    void on_browserButton_clicked();

private:
    Ui::IPSetupWindow *ui;
    QHeaderView *m_viewHeader;
    IPSetup *m_finder;
    IPSetupTreeModel *m_devModel;
    bool m_clearModelNext;
    QList<NBurnDevice> m_devices;
    QList<NBurnDevice> m_displayedDevices;
    ConfigRecord m_cf;

    virtual void keyPressEvent(QKeyEvent * event);
    void SetTreeColumnWidths();
    void DoInputUpdate(const QModelIndex &index);
    void SetWifiInputState(bool state);
};

inline bool MacEqual(const quint8 *lhs, const quint8 *rhs) {
    return (lhs[0]==rhs[0])
            && (lhs[1]==rhs[1])
            && (lhs[2]==rhs[2])
            && (lhs[3]==rhs[3])
            && (lhs[4]==rhs[4])
            && (lhs[5]==rhs[5]);
}

#endif // IPSETUPWINDOW_H
